# frozen_string_literal: true

require 'lace/package'
require 'lace/exceptions'
require 'fileutils'

module Lace
  module_function

  def apply
    package_name = ARGV.shift
    raise ResourceNotSpecified unless package_name
    ohai "Applying package #{package_name}"
    package = Package.new(package_name, ARGV.first)

    diff = PackageUtils.diff(package_name)

    if diff.added.empty? && diff.removed.empty?
      puts 'Everything is already in sync. Nothing to do.'
      return
    end

    diff.added.each do |f|
      dest = f.as_dotfile(Dir.home, package.path)
      dest_path = dest.to_path.gsub(Dir.home, '$HOME')
      puts "Link #{f.basename} to #{dest_path}?"
      next unless ARGV.confirm?("Link #{f.basename} to #{dest_path}?")

      package.link_file_or_directory(f, dest, force: true)
      ohai "Linked"
    end

    diff.removed.each do |f|
      dest = f.as_dotfile(Dir.home, package.path)
      dest_path = dest.to_path.gsub(Dir.home, '$HOME')
      next unless ARGV.confirm?("Remove link for #{f.basename} from #{dest_path}?")

      package.unlink_file(f, force: true)
      ohai "Removed"
    end
  end
end